/*
 * Header File < grafplot.h >
 *
 * The Graph Plotting Routines for Borland Graphics Interface
 *
 * Copyright (c) 1996 Taiji Yamada
 */
#ifndef _GRAFPLOT_H_
#define _GRAFPLOT_H_

#ifndef Float
#define Float double
#endif

/*
 * ץåȤμ
 */
#define PLOT_LINE   (1L<<0)
#define PLOT_DOT    (1L<<1)
#define PLOT_POINT  (1L<<2) /* reserved future */
#define PLOT_ERROR  (1L<<3) /* reserved future */
/*
 * 顼μ
 */
enum plotcolor_kinds {
  COLOR_RED,
  COLOR_GREEN,
  COLOR_BLUE,
  COLOR_MAGENTA,
  COLOR_YELLOW,
  COLOR_CYAN,
  N_COLOR_KINDS,
};
/*
 * 饤μ
 */
enum plotline_kinds {
  LINE_SOLID,
  LINE_DOTTED,
  LINE_CENTER,
  LINE_DASHED,
  N_LINE_KINDS,
};
/*
 * ݥȤμ
 */
enum plotpoint_kinds {
  POINT_CIRCLE,
  POINT_CROSS,
  POINT_RECTANGLE,
  POINT_TIMES,
  POINT_TRIANGLE,
  N_POINT_KINDS,
};
/*
 * 󥰤μ
 */
enum scale_kinds {
  SCALE_DEC,
  SCALE_LOG10,
};
/*
 * źΥܥ
 */
enum axis_index {
  ERROR_H = -2, /* these indicate error elements of error bar */
  ERROR_L = -1,
  AXIS_C = 0, /* indicates another axis with color gradations */
  AXIS_X,
  AXIS_Y,
  AXIS_Z,
  AXIS_W, /* over 4 dimension needs direct index */
};
/*
 * 2դμ
 */
enum r2_kinds {
  R2_NORMAL,
  R2_LOGX,
  R2_LOGY,
  R2_LOGLOG,
};
/*
 * 3դμ
 */
enum r3_kinds {
  R3_ROTATION,
  R3_PERSPECTIVE,
};
/*
 * ܤμξ
 */
struct plotaxistype {
  char *title;
  int x, y;
  int scale_type;
  Float from_range;
  Float to_range;
  int axis_flag;
  int tick_flag;
  int zero_flag;
  Float from_tick, to_tick;
  int div_tick;
  int tick_height;
  int tick_offset;
};
/*
 * ץåоݤΥ٥ȥ륿
 */
struct plotvectortype {
  char *title;
  int x, y;
  int nl, nh;
  Float *data;
  int n_axes_are;
  int *axes_are;
  unsigned long plotkind;
  int colorkind;
  int linekind;
  int pointkind;
};
/*
 * ץåоݤΥޥȥå
 */
struct plotmatrixtype {
  char *title;
  int x, y;
  int nrl, nrh, ncl, nch;
  Float **data;
  int n_axes_are;
  int *axes_are;
  unsigned long plotkind;
  int colorkind;
  int linekind;
  int pointkind;
};
/*
 * ץåоݤΥޥȥåT
 */
struct plotmatrixTtype {
  char *title;
  int x, y;
  int nrl, nrh, ncl, nch;
  Float **data;
  int n_axes_are;
  int *axes_are;
  unsigned long plotkind;
  int colorkind;
  int linekind;
  int pointkind;
};
/*
 * Τξ
 */
struct plotgraphtype {
  char *title;
  int x, y;
  int color_kind_flag;
  int color_axis_flag;
  int palette_from, palette_to;
  int r3kind;
  int tick_font, tick_fontsize;
  int n_axes;
  struct plotaxistype *axes;
  int n_data;
  int n_vectors;
  struct plotvectortype *vectors;
  int n_matrices;
  struct plotmatrixtype *matrices;
  int n_matricesT;
  struct plotmatrixTtype *matricesT;
};

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

/*
 * Τؿ
 */
/* եץåȤΥȥ */
extern void set_plotgraph_title( char *title );

/* եץåȤΥȥȼ */
extern void set_plotgraph( char *title, int dimension );

/* եץåȤǥǡ򥫥顼Ƕ̤뤫 */
extern void set_plotgraph_color_kind( int color_kind_flag );

/* եץåȤǤ⤦켡򥫥顼ɽ뤫 */
extern void set_plotgraph_color_axis( int color_axis_flag );

/* եץåȤ2դμ */
extern void set_plotgraph_r2kind( int r2kind );

/* եץåȤ3դμ */
extern void set_plotgraph_r3kind( int r3kind );

/* եץåȤΥꥻå */
extern void reset_plotgraph( void );

/*
 * ץåȼ˴ؤؿ
 */
/* եץåȤμ */
extern void set_plotgraph_n_axes( int n_axes );

/* ץåȼΥȥȥ󥰥 */
extern void set_ith_plotaxis( int ith, char *title, int scale_type );

#define set_x_plotaxis(title,scale_type) \
        set_ith_plotaxis(AXIS_X,title,scale_type)
#define set_y_plotaxis(title,scale_type) \
        set_ith_plotaxis(AXIS_Y,title,scale_type)
#define set_z_plotaxis(title,scale_type) \
        set_ith_plotaxis(AXIS_Z,title,scale_type)
#define set_w_plotaxis(title,scale_type) \
        set_ith_plotaxis(AXIS_W,title,scale_type)
#define set_c_plotaxis(title,scale_type) \
        set_ith_plotaxis(AXIS_C,title,scale_type)

/* ץåȼΥȥ */
extern void set_ith_plotaxis_title( int ith, char *title );

#define set_x_plotaxis_title(title) \
        set_ith_plotaxis_title(AXIS_X,title)
#define set_y_plotaxis_title(title) \
        set_ith_plotaxis_title(AXIS_Y,title)
#define set_z_plotaxis_title(title) \
        set_ith_plotaxis_title(AXIS_Z,title)
#define set_w_plotaxis_title(title) \
        set_ith_plotaxis_title(AXIS_W,title)
#define set_c_plotaxis_title(title) \
        set_ith_plotaxis_title(AXIS_C,title)

/* ץåȼΥ󥰥 */
extern void set_ith_plotaxis_scale_type( int ith, int scale_type );

#define set_x_plotaxis_scale_type(scale_type) \
        set_ith_plotaxis_scale_type(AXIS_X,scale_type)
#define set_y_plotaxis_scale_type(scale_type) \
        set_ith_plotaxis_scale_type(AXIS_Y,scale_type)
#define set_z_plotaxis_scale_type(scale_type) \
        set_ith_plotaxis_scale_type(AXIS_Z,scale_type)
#define set_w_plotaxis_scale_type(scale_type) \
        set_ith_plotaxis_scale_type(AXIS_W,scale_type)
#define set_c_plotaxis_scale_type(scale_type) \
        set_ith_plotaxis_scale_type(AXIS_C,scale_type)

/* ץåȼΥե饰(缴,å,ݤ) */
extern
void set_ith_plotaxis_flags( int ith,
                            int axis_flag, int tick_flag, int zero_flag );

#define set_x_plotaxis_flags(axis_flag,tick_flag,zero_flag) \
        set_ith_plotaxis_flags(AXIS_X,axis_flag,tick_flag,zero_flag)
#define set_y_plotaxis_flags(axis_flag,tick_flag,zero_flag) \
        set_ith_plotaxis_flags(AXIS_Y,axis_flag,tick_flag,zero_flag)
#define set_z_plotaxis_flags(axis_flag,tick_flag,zero_flag) \
        set_ith_plotaxis_flags(AXIS_Z,axis_flag,tick_flag,zero_flag)
#define set_w_plotaxis_flags(axis_flag,tick_flag,zero_flag) \
        set_ith_plotaxis_flags(AXIS_W,axis_flag,tick_flag,zero_flag)
#define set_c_plotaxis_flags(axis_flag,tick_flag,zero_flag) \
        set_ith_plotaxis_flags(AXIS_C,axis_flag,tick_flag,zero_flag)

/* Υ */
extern
void set_ith_plotaxis_range( int ith, Float from_range, Float to_range );

#define set_x_plotaxis_range(from_range,to_range) \
        set_ith_plotaxis_range(AXIS_X,from_range,to_range)
#define set_y_plotaxis_range(from_range,to_range) \
        set_ith_plotaxis_range(AXIS_Y,from_range,to_range)
#define set_z_plotaxis_range(from_range,to_range) \
        set_ith_plotaxis_range(AXIS_Z,from_range,to_range)
#define set_w_plotaxis_range(from_range,to_range) \
        set_ith_plotaxis_range(AXIS_W,from_range,to_range)
#define set_c_plotaxis_range(from_range,to_range) \
        set_ith_plotaxis_range(AXIS_C,from_range,to_range)

/* Υå */
extern
void set_ith_plotaxis_tick( int ith,
                           Float from_tick, Float to_tick, int div_tick );

#define set_x_plotaxis_tick(from_tick,to_tick,div_tick) \
        set_ith_plotaxis_tick(AXIS_X,from_tick,to_tick,div_tick)
#define set_y_plotaxis_tick(from_tick,to_tick,div_tick) \
        set_ith_plotaxis_tick(AXIS_Y,from_tick,to_tick,div_tick)
#define set_z_plotaxis_tick(from_tick,step_tick,div_tick) \
        set_ith_plotaxis_tick(AXIS_Z,from_tick,to_tick,div_tick)
#define set_w_plotaxis_tick(from_tick,to_tick,div_tick) \
        set_ith_plotaxis_tick(AXIS_W,from_tick,to_tick,div_tick)
#define set_c_plotaxis_tick(from_tick,to_tick,div_tick) \
        set_ith_plotaxis_tick(AXIS_C,from_tick,to_tick,div_tick)

/* ץåȼΥꥻå */
extern void reset_ith_plotaxis( int ith );

extern void reset_x_plotaxis( void );
extern void reset_y_plotaxis( void );
extern void reset_z_plotaxis( void );
extern void reset_w_plotaxis( void );
extern void reset_c_plotaxis( void );

/*
 * ץåȥ٥ȥ˴ؤؿ
 */
/* ץåȥ٥ȥο */
extern void set_plotgraph_n_vectors( int n_vectors );

/* ץåȥ٥ȥΥȥ */
extern void set_ith_plotvector_title( int ith, char *title );

#define set_plotvector_title(title) \
        set_ith_plotvector_title(1,title)

/* ץåȥ٥ȥΥǡ */
extern void set_ith_plotvector_data( int ith, int nl, int nh, Float *data );

#define set_plotvector_data(title,nl,nh,data) \
        set_ith_plotvector_data(1,title,nl,nh,data)

/* ץåȥ٥ȥΥȥȥǡ */
extern
void set_ith_plotvector( int ith, char *title,
                        int nl, int nh, Float *data );

#define set_plotvector(title,nl,nh,data) \
        set_ith_plotvector(1,title,nl,nh,data)

/* ץåȥ٥ȥμб */
extern void set_ith_plotvector_axis( int ith, int axis, int index );

#define set_plotvector_axis(axis,index) \
        set_ith_plotvector_axis(1,axis,index)

/* ץåȥ٥ȥͤμ */
extern
Float ith_plotvector_axis_value( int ith_vector,
                                int ith_axis, int ith );

#define plotvector_ith_axis_ith_value(ith_axis,ith) \
        ith_plotvector_axis_value(1,ith_axis,ith)

/* ץåȥ٥ȥΥץåȼ */
extern
void set_ith_plotvector_kinds( int ith, unsigned long plotkind,
                              int colorkind, int linekind, int pointkind );

#define set_plotvector_kinds(plotkind,colorkind,linekind,pointkind) \
        set_ith_plotvector_kinds(1,plotkind,colorkind,linekind,pointkind)

/* ץåȥ٥ȥΥꥻå */
extern void reset_ith_plotvector( int ith );

extern void reset_plotvector( void );

/*
 * ץåȥޥȥå˴ؤؿ
 */
/* ץåȥޥȥåο */
extern void set_plotgraph_n_matrices( int n_matrices );

/* ץåȥޥȥåΥȥ */
extern void set_ith_plotmatrix_title( int ith, char *title );

#define set_plotmatrix_title(title) \
        set_ith_plotmatrix_title(1,title)

/* ץåȥޥȥåΥǡ */
extern
void set_ith_plotmatrix_data( int ith,
                             int nrl, int nrh, int ncl, int nch,
                             Float **data );

#define set_plotmatrix_data(nrl,nrh,ncl,nch,data) \
        set_ith_plotmatrix_data(1,nrl,nrh,ncl,nch,data)

/* ץåȥޥȥåΥȥȥǡ */
extern
void set_ith_plotmatrix( int ith, char *title,
                        int nrl, int nrh, int ncl, int nch, Float **data );

#define set_plotmatrix(title,nrl,nrh,ncl,nch,data) \
        set_ith_plotmatrix(1,title,nrl,nrh,ncl,nch,data)

/* ץåȥޥȥåμб */
extern void set_ith_plotmatrix_axis( int ith, int axis, int index );

#define set_plotmatrix_axis(axis,index) \
        set_ith_plotmatrix_axis(1,axis,index)

/* ץåȥޥȥåͤμ */
extern
Float ith_plotmatrix_axis_value( int ith_matrix,
                                int ith_axis, int ith );

#define plotmatrix_axis_value(ith_axis,ith) \
        ith_plotmatrix_axis_value(1,ith_axis,ith)

/* ץåȥޥȥåΥץåȼ */
extern
void set_ith_plotmatrix_kinds( int ith, unsigned long plotkind,
                              int colorkind, int linekind, int pointkind );

#define set_plotmatrix_kinds(plotkind,colorkind,linekind,pointkind) \
        set_ith_plotmatrix_kinds(1,plotkind,colorkind,linekind,pointkind)

/* ץåȥޥȥåΥꥻå */
extern void reset_ith_plotmatrix( int ith );

extern void reset_plotmatrix( void );

/*
 * ץåȥޥȥåT˴ؤؿ
 */
/* ץåȥޥȥåTο */
extern void set_plotgraph_n_matricesT( int n_matricesT );

/* ץåȥޥȥåΥȥ */
extern void set_ith_plotmatrixT_title( int ith, char *title );

#define set_plotmatrixT_title(title) \
        set_ith_plotmatrixT_title(1,title)

/* ץåȥޥȥåΥǡ */
extern
void set_ith_plotmatrixT_data( int ith,
                              int nrl, int nrh, int ncl, int nch,
                              Float **data );

#define set_plotmatrixT_data(nrl,nrh,ncl,nch,data) \
        set_ith_plotmatrixT_data(1,nrl,nrh,ncl,nch,data)

/* ץåȥޥȥåΥȥȥǡ */
extern
void set_ith_plotmatrixT( int ith, char *title,
                        int nrl, int nrh, int ncl, int nch, Float **data );

#define set_plotmatrixT(title,nrl,nrh,ncl,nch,data) \
        set_ith_plotmatrixT(1,title,nrl,nrh,ncl,nch,data)

/* ץåȥޥȥåTμб */
extern void set_ith_plotmatrixT_axis( int ith, int axis, int index );

#define set_plotmatrixT_axis(axis,index) \
        set_ith_plotmatrixT_axis(1,axis,index)

/* ץåȥޥȥåTͤμ */
extern
Float ith_plotmatrixT_axis_value( int ith_matrixT,
                                 int ith_axis, int ith );

#define plotmatrixT_axis_value(ith_axis,ith) \
        ith_plotmatrixT_axis_value(1,ith_axis,ith)

/* ץåȥޥȥåTΥץåȼ */
extern
void set_ith_plotmatrixT_kinds( int ith, unsigned long plotkind,
                               int colorkind, int linekind, int pointkind );

#define set_plotmatrixT_kinds(plotkind,colorkind,linekind,pointkind) \
        set_ith_plotmatrixT_kinds(1,plotkind,colorkind,linekind,pointkind)

/* ץåȥޥȥåTΥꥻå */
extern void reset_ith_plotmatrixT( int ith );

extern void reset_plotmatrixT( void );

/*
 * ꤵ줿Ƥξ褹
 */
/* 顼 */
extern void plotgraph_color_axis( void );

/* 2դμ */
extern void plotgraph_2d_axes( void );

/* 2դΥǡ */
extern void plotgraph_2d_data( void );

/* 2դ */
extern void plotgraph_2d( void );

/* 3դμ */
extern void plotgraph_3d_axes( void );

/* 3դΥǡ */
extern void plotgraph_3d_data( void );

/* 3դ */
extern void plotgraph_3d( void );

/* nդμ */
extern void plotgraph_nd_axes( void );

/* nդΥǡ */
extern void plotgraph_nd_data( void );

/* nդ */
extern void plotgraph_nd( void );

/* ꤵ줿 */
extern void plotgraph( void );

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif

#endif
