/*
 * Source File < gsave.c >
 *
 * Utility to Save Screen for BGI and XBGI by TIFF format
 *
 * Copyright (c) 1992-94 Tokyo Denki University, Taiji Yamada
 */
#include <stdio.h>
#include <gsave.h>
#include "internal.h"

int savegraphtotiff( int mode, char *filename )
{
  unsigned int header_size = 198;
  unsigned char header[] = {
    0x49, 0x49, 0x2A, 0x00, 0x08, 0x00, 0x00, 0x00,
    0x0E, 0x00, 0xFF, 0x00, 0x03, 0x00, 0x01, 0x00,
    0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01,
    0x03, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x02, /* header[30] = width */
    0x00, 0x00, 0x01, 0x01, 0x03, 0x00, 0x01, 0x00,
    0x00, 0x00, 0x90, 0x01, 0x00, 0x00, 0x02, 0x01, /* header[42] = height */
    0x03, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x00,
    0x00, 0x00, 0x03, 0x01, 0x03, 0x00, 0x01, 0x00,
    0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x06, 0x01,
    0x03, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x00,
    0x00, 0x00, 0x11, 0x01, 0x04, 0x00, 0x01, 0x00,
    0x00, 0x00, 0xC6, 0x00, 0x00, 0x00, 0x12, 0x01,
    0x03, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x00,
    0x00, 0x00, 0x15, 0x01, 0x03, 0x00, 0x01, 0x00,
    0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x16, 0x01,
    0x04, 0x00, 0x01, 0x00, 0x00, 0x00, 0x90, 0x01, /* offset 0x90->0xe0 */
    0x00, 0x00, 0x17, 0x01, 0x04, 0x00, 0x01, 0x00,
    0x00, 0x00, 0x00, 0x7d, 0x00, 0x00, 0x19, 0x01,
    0x03, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x00,
    0x00, 0x00, 0x1A, 0x01, 0x05, 0x00, 0x01, 0x00,
    0x00, 0x00, 0xB6, 0x00, 0x00, 0x00, 0x1B, 0x01,
    0x05, 0x00, 0x01, 0x00, 0x00, 0x00, 0xBE, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xF0, 0x00,
    0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0xF0, 0x00,
    0x00, 0x00, 0x01, 0x00, 0x00, 0x00,
  };
  int i, j, k, width = getmaxx()+1, height = getmaxy()+1;
  int bkcolor, c;
  unsigned char pix;
  FILE *fp;
#ifdef _XBGI_
#include <graphix.h>
  extern xbgi *XBGI;
  XImage *image;
  unsigned char *data;
#endif

  if ( (fp=fopen( filename, "wb" )) == NULL ){
    fprintf( stderr, "can't open file: %s \n", filename );
    return( -1 );
  }
  header[30] = width%256;  header[31] = width/256;
  header[42] = height%256; header[43] = height/256;
  fwrite( (void*)header, sizeof(char), header_size, fp );
  switch ( mode ){
  case GSAVE_MONO:
  case GSAVE_MONO_REVERSE:
    bkcolor = getbkcolor();
#ifdef _XBGI_
    setactivepage( XBGI->vpage );
    image = XGetImage( XBGIDisplay, XBGIWins[XBGI->apage],
                      0, 0, width, height, AllPlanes, ZPixmap );
    data = (unsigned char *)image->data;
    for ( i=0; i<height; i++ ){
      for ( k=j=0; j<image->bytes_per_line; j++, data++ ){
        if ( j<image->xoffset ) continue;
        k++;
        if ( width < k ) continue;
        if ( k%8 == 0 ) pix = 0;
        c = XBGIColortable[*data];
        pix = pix|((c!=bkcolor)?(0x80>>(k%8)):0);
        if ( k%8 == 7 ){
          if ( mode == GSAVE_MONO_REVERSE ) pix = 0xFF^pix;
          fwrite( &pix, sizeof(char), 1, fp );
        }
      }
    }
    XDestroyImage( image );
#else
    for ( i=0; i<height; i++ ){
      for ( j=0; j<width; j+=8 ){
        pix = 0;
        for ( k=0; k<8; k++ ){
          c = getpixel( j+k, i );
          pix = pix|((c!=bkcolor)?(0x80>>k):0);
        }
        if ( mode == GSAVE_MONO_REVERSE ) pix = 0xFF^pix;
        fwrite( &pix, sizeof(char), 1, fp );
      }
    }
#endif
    break;
  }
  fclose( fp );
  return( 0 );
}



