/*
 * Source File < ps/misc.c >
 *
 * Misc Functions for PS: Borland Graphics Interface for PostScript
 *
 * Copyright (c) 1992-94,95 Tokyo Denki University, Taiji Yamada
 */
#include "ps_p.h"
#include "script/prolog.def"
#include "script/trailer.def"
#include "script/begin.def"
#include "script/end.def"

#ifndef DefaultInterpName
#  define DefaultInterpName "gs -"
#endif
#ifndef DefaultFileName
#  define DefaultFileName "scratch.ps"
#endif

void ps_xbgi_flush( void )
{
  if ( PSBGIInterpP ) fflush( PSBGIInterpP );
  if ( PSBGIFileP   ) if ( PSBGIFileP == stdout ) fflush( PSBGIFileP );
}
void ps_xbgi_vprintf( char *fmt, va_list argptr )
{
  if ( PSBGIInterpP ) vfprintf( PSBGIInterpP, fmt, argptr );
  if ( PSBGIFileP   ) vfprintf( PSBGIFileP,   fmt, argptr );
}
void ps_xbgi_printf( char *fmt, ... )
{
  va_list argptr;
  
  va_start( argptr, fmt );
  ps_xbgi_vprintf( fmt, argptr );
  va_end( argptr );
}
void ps_xbgi_open( void )
{
  char buf[256];
  time_t now;
  struct tm *now_tm;
  char tm_stamp[64];

  time( &now );
  now_tm = localtime( &now );
  strftime( tm_stamp, 64, "%a %h %d %T %Y", now_tm );

  if ( PSBGIUseInterp ){
    if ( strcmp(PSBGIInterpName,"")==0 )
      strcpy( PSBGIInterpName, DefaultInterpName );
    PSBGIInterpP = popen( PSBGIInterpName, "w" );
    if ( PSBGIInterpP == NULL ){
      sprintf( buf, "cannot open pipe (%s).", PSBGIInterpP );
      if ( !PSBGIUseFile )
        FatalError( buf );
      else{
        Warning( buf );
        PSBGIInterpP = NULL;
        PSBGIUseInterp = 0;
      }
    }
  }else{
    PSBGIInterpP = NULL;
  }
  if ( PSBGIUseFile ){
    if ( strcmp(PSBGIFileName,"")==0 )
      strcpy( PSBGIFileName, DefaultFileName );
    PSBGIFileP = fopen( PSBGIFileName, "w" );
    if ( PSBGIFileP == NULL ){
      sprintf( buf, "cannot open file (%s).", PSBGIFileP );
      if ( !PSBGIUseInterp )
        FatalError( buf );
      else{
        Warning( buf );
        PSBGIFileP = NULL;
        PSBGIUseFile = 0;
      }
    }
  }else{
    PSBGIFileP = stdout;
    PSBGIUseFile = !0;
    strcpy( PSBGIFileName, "(stdout)" );
  }
  ps_xbgi_printf( "%%!\n" );
  ps_xbgi_printf( "%%%%BoundingBox: %d %d %d %d\n",
                 BBX, BBY, BBX+PSBGIRoot.width, BBY+PSBGIRoot.height );
  ps_xbgi_printf( "%%%%Title: %s\n", PSBGITitle );
  ps_xbgi_printf( "%%%%CreationDate: %s\n", tm_stamp );
  ps_xbgi_printf( "%%%%Creator: %s\n",
                 "XBGI(PS) by Taiji Yamada (taiji@henon.d.dendai.ac.jp)" );
  if ( PSBGIEncapsule )
    ps_xbgi_printf( "%%%%Pages: %d\n", 1 );
  else
    ps_xbgi_printf( "%%%%Pages: (attend)\n" );
  ps_xbgi_printf( "%%%%EndComments\n" );
  ps_xbgi_printf( PROLOG );
}
void ps_xbgi_close( void )
{
  cleardevice();
  ps_xbgi_printf( TRAILER );
  if ( PSBGIInterpP ) pclose( PSBGIInterpP );
  if ( PSBGIFileP   ) if ( PSBGIFileP != stdout ) fclose( PSBGIFileP );
}
void ps_xbgi_put_color( int color )
{
  Color rgb = PSBGIRGB[XBGI->palette.colors[color]];
  
  switch ( XBGI->gmode ){
  case PSMONO:
    if ( color != XBGI->bkcolor )
      ps_xbgi_printf( "0 setgray\n" );
    else
      ps_xbgi_printf( "1 setgray\n" );
    break;
  case PSGRAY:
    ps_xbgi_printf( "%g setgray\n", (rgb.red+rgb.green+rgb.blue)/3 );
    break;
  case PSCOLOR:
  default:
    ps_xbgi_printf( "%g %g %g setrgbcolor\n", rgb.red, rgb.green, rgb.blue );
    break;
  }
}
void ps_xbgi_begin( char *name )
{
  if ( !PSBGIRoot.drawn ){
    ps_xbgi_printf( "%%%%Page: %d %d\n",
		   PSBGIRoot.showpage_count+1, PSBGIRoot.showpage_count+1 );
    ps_xbgi_printf( "%%%%PageBoundingBox: %d %d %d %d\n",
		   BBX, BBY, BBX+PSBGIRoot.width, BBY+PSBGIRoot.height );
    ps_xbgi_printf( BEGIN );
  }
  ps_xbgi_printf( "%% begin %s\n", name );
}
void ps_xbgi_end( char *name )
{
  ps_xbgi_printf( "%% end %s\n", name );
  ps_xbgi_flush();
  PSBGIRoot.drawn = !0;
}
void ps_xbgi_showpage( void )
{
  if ( PSBGIRoot.drawn ){
    ps_xbgi_printf( END );
    if ( !PSBGIEncapsule )
      ps_xbgi_printf( "showpage\n" );
    PSBGIRoot.drawn = 0;
    PSBGIRoot.showpage_count++;
  }
}
void ps_xbgi_put_string( char *string, int len )
{
  int i;

  for ( i=0; i<len; i++ ){
    switch( string[i] ){
    case '\\':
    case '(':
    case ')':
      ps_xbgi_printf( "\\%c", string[i] );
      break;
    default:
      ps_xbgi_printf( "%c", string[i] );
      break;
    }
  }
}


