/*
 * Source File < x11/color.c >
 *
 * Standard Functions: Borland Graphics Interface for Xlib
 *
 * Copyright (c) 1992-94,95 Tokyo Denki University, Taiji Yamada
 */
#include "x11_p.h"
char x11_xbgi_scolor_names[MAXCOLORS+1][64] = {
#include "colors/scolor.def"
};

/*
 * standard functions to get and set color
 */
int getcolor( void )
{
  return XBGI->color;
}
int getbkcolor( void )
{
  return XBGI->bkcolor;
}
int getmaxcolor( void )
{
  return XBGI->palette.size-1;
}
void setcolor( int color )
{
  XBGI->color = minmax( 0, color, XBGI->palette.size-1 );
  XSetForeground( XBGIDisplay, XBGIDrawGC, XBGIPixels[XBGI->color] );
}
void setbkcolor( int color )
{
  XBGI->bkcolor = minmax( 0, color, XBGI->palette.size-1 );
  XSetWindowBackground( XBGIDisplay, XBGIWins[XBGI->apage], 
		       XBGIPixels[XBGI->bkcolor] );
  XSetBackground( XBGIDisplay, XBGIDrawGC,  XBGIPixels[XBGI->bkcolor] );
  XSetBackground( XBGIDisplay, XBGIFillGC,  XBGIPixels[XBGI->bkcolor] );
  XSetBackground( XBGIDisplay, XBGIPixelGC, XBGIPixels[XBGI->bkcolor] );
  XSetBackground( XBGIDisplay, XBGIImageGC, XBGIPixels[XBGI->bkcolor] );
  /*XClearWindow( XBGIDisplay, XBGIWins[XBGI->apage] );*/
}
void setmaxcolor( int c )
{
  int i, cc;
  unsigned long plane_mask;
  struct rgbtype {
    unsigned short red, green, blue;
  } rgb[8] = {
    { 0, 0, 0 },
    { 0, 0, 1 },
    { 0, 1, 0 },
    { 0, 1, 1 },
    { 1, 0, 0 },
    { 1, 0, 1 },
    { 1, 1, 0 },
    { 1, 1, 1 },
  };
  XColor exact, color;
  Status ok;

  switch( XBGI->gmode ){
  case X11MONO:
  case X11SGRAY: /* It's scamping. */
    XBGI->palette.size = 2;
    XBGIColortable[BlackPixel(XBGIDisplay,XBGIScreen)] = BLACK;
    XBGIColortable[WhitePixel(XBGIDisplay,XBGIScreen)] = WHITE;
    XBGIPixels[BLACK] = BlackPixel( XBGIDisplay, XBGIScreen );
    XBGIPixels[WHITE] = WhitePixel( XBGIDisplay, XBGIScreen );
    for ( i=0; i<XBGI->palette.size; i++ )
      XBGI->palette.colors[i] = i;
    break;
  case X11GRAY: /* I don't know grayscale visual class. */
    XBGI->palette.size = c + 1;
    ok = XAllocColorCells( XBGIDisplay, XBGIColormap,
                          False, &plane_mask, False,
                          XBGIPixels, XBGI->palette.size );
    if ( ok == False ) FatalError( "gray-scale cell allocation error" );
    for ( i=0; i<XBGI->palette.size; i++ ){
      XBGIColortable[XBGIPixels[i]] = i;
      XBGI->palette.colors[i] = i;
    }
    for ( i=1; i<XBGI->palette.size; i++ )
      setrgbpalette( i, i, i, i );
    setallpalette( &XBGI->palette );
    break;
  case X11SCOLOR: /* It's supported since version 2.2. */
    if ( c<1 || 255<c ) return;
    XBGI->palette.size = c + 1;
    for ( i=0; i<XBGI->palette.size; i++ ){
      ok = XAllocNamedColor( XBGIDisplay, XBGIColormap,
                            x11_xbgi_scolor_names[i], &exact, &color );
      if ( ok == False ) Warning( "named color allocation fault" );
      else{
        XBGIPixels[i] = exact.pixel;
        XBGIColortable[XBGIPixels[i]] = i;
        XBGI->palette.colors[i] = i;
      }
    }
    break;
  case X11COLOR:
    if ( c<1 || 255<c ) return;
    if ( XBGI->gerror != grNoInitGraph )
      XFreeColors( XBGIDisplay, XBGIColormap, XBGIPixels,
                  XBGI->palette.size, 0 );
    XBGI->palette.size = c + 1;
    ok = XAllocColorCells( XBGIDisplay, XBGIColormap,
                          False, &plane_mask, False,
                          XBGIPixels, XBGI->palette.size );
    if ( ok == False ){
      /* 1st failed */
      XBGIColormap = XCreateColormap( XBGIDisplay, XBGIRoot, XBGIVisual,
                                       AllocNone );
      /*XInstallColormap( XBGIDisplay, XBGIColormap );*/ 
      XSetWindowColormap( XBGIDisplay, XBGIRoot, XBGIColormap );
      ok = XAllocColorCells( XBGIDisplay, XBGIColormap,
                            False, &plane_mask, False,
                            XBGIPixels, XBGI->palette.size );
      /* need technique for settings near the standard color map */
      if ( ok == False ) FatalError( "color cell allocation fault" );
    }
    for ( i=0; i<XBGI->palette.size; i++ ){
      XBGIColortable[XBGIPixels[i]] = i;
      XBGI->palette.colors[i] = i;
    }
    if ( XBGI->colornames && XBGI->gerror == grNoInitGraph ){
      /* color name setting at graph booting */
      for ( i=0; i<XBGI->palette.size; i++ ){
        ok = XParseColor( XBGIDisplay, XBGIColormap,
                         x11_xbgi_scolor_names[i], &color );
        if ( ok == False ) Warning( "named color storing fault" );
        else
          _setrgbpalette( i, color.red, color.green, color.blue );
      }
    }else{
      if ( XBGI->palette.size>=8 ){
	/* normal algorithum color setting */
	for ( i=0; i<XBGI->palette.size/8; i++ )
	  for ( cc=0; cc<8; cc++ )
	    setrgbpalette( 8*i+cc,
			  8*(i+1)*rgb[cc].red,
			  8*(i+1)*rgb[cc].green,
			  8*(i+1)*rgb[cc].blue );
	for ( i=1; i<XBGI->palette.size/8; i++ )
	  setrgbpalette( 8*i+0, 4*(i+1), 4*(i+1), 4*(i+1) );
	cc = XBGI->palette.size/2;
	for ( i=0; i<XBGI->palette.size/8; i++ )
	  setrgbpalette( 8*i+7, cc+4*(i+1), cc+4*(i+1), cc+4*(i+1) );
      }else{
	/* case less than 8 color */
	_setrgbpalette( 0, 0, 0, 0 );
	for ( i=1; i<XBGI->palette.size-1; i++ ){
	  _setrgbpalette( i, 65536-1, 65536-1, 65536-1 );
	}
	_setrgbpalette( 1, 65536-1, 65536-1, 65536-1 );
      }
    }
    break;
  }
}









