/*
 * Source File < x11/driver.c >
 *
 * Standard Functions: Borland Graphics Interface for Xlib
 *
 * Copyright (c) 1992-94,95 Tokyo Denki University, Taiji Yamada
 */
#include "x11_p.h"
#include "option.h"

void detectgraph( int *graphdriver, int *graphmode )
{
  /*
   * case of detecting before initgraph, always X11 and X11COLOR
   */
  if ( XBGIS == NULL || XBGI->inherit_members == NULL ){
    *graphdriver = X11;
    *graphmode   = X11COLOR;
    return;
  }
  /*
   * case of detecting after initgraph, research visual class and depth
   */
  switch( *graphdriver ){
  case DETECT: default:
    *graphdriver = XBGI->gdriver = X11;
    switch ( XBGIVisualClass ){
    case StaticGray:
      *graphmode = XBGI->gmode = X11SGRAY;
      if ( XBGIDepth == 1 ){
        *graphmode = XBGI->gmode = X11MONO;
        XBGI->ncolor = 2;
        XBGI->palette.size = (unsigned char)XBGI->ncolor-1;
      }
      break;
    case GrayScale:
      *graphmode = XBGI->gmode = X11GRAY;
      if ( XBGIDepth == 1 ){
        *graphmode = XBGI->gmode = X11MONO;
        XBGI->ncolor = 2;
        XBGI->palette.size = (unsigned char)XBGI->ncolor-1;
      }
      break;
    case StaticColor:
      *graphmode = XBGI->gmode = X11SCOLOR;
      break;
    case PseudoColor:
    case TrueColor:
    case DirectColor:
      *graphmode = XBGI->gmode = X11COLOR;
      break;
    }
    break;
  case X11  :
  case PS   :
  case TCLTK:
  case TGIF :
    *graphmode = XBGI->gmode;
    /* no check */
    break;
  }
}
/*
 * default special driver, destroyer and creater
 */
void _driver( void )
{
  XUnmapSubwindows( XBGIDisplay, XBGIRoot );
  XUnmapWindow( XBGIDisplay, XBGIRoot );
  XFlush( XBGIDisplay );

  XFreeGC( XBGIDisplay, XBGIDrawGC  );
  XFreeGC( XBGIDisplay, XBGIFillGC  );
  XFreeGC( XBGIDisplay, XBGIPixelGC );
  XFreeGC( XBGIDisplay, XBGIImageGC );

  XFreeColormap( XBGIDisplay, XBGIColormap );
  if ( XBGIFontStruct ) XFreeFont( XBGIDisplay, XBGIFontStruct );

  XDestroySubwindows( XBGIDisplay, XBGIRoot );
  XDestroyWindow( XBGIDisplay, XBGIRoot );

#if 0 /* for that events are remaining */
  XSync( XBGIDisplay, False );
  XCloseDisplay( XBGIDisplay );
#endif
#if 0 /* for segmentation fault bug? */
  free( XBGI->inherit_members );
  XBGI->inherit_members = NULL;
#endif
}
void driver( void )
{
#if 0 /* it's bug */
#include "bitmaps/icons/xbgi_a.xbm"
#include "bitmaps/icons/xbgi_b.xbm"
#include "bitmaps/icons/xbgi_c.xbm"
  struct xbm {
    char *bits;
    int width, height;
  } xbms[] = {
    { xbgi_c_bits, xbgi_c_width, xbgi_c_height },
    { xbgi_b_bits, xbgi_b_width, xbgi_b_height },
    { xbgi_a_bits, xbgi_a_width, xbgi_a_height },
    { NULL, 0, 0 }
  };
#endif
  int i, j, c, flag_mask, status;
  XSizeHints def;
  XSetWindowAttributes att;
  XEvent ev;
  XColor fgexact, fgcolor, bgexact, bgcolor, lcolor, dcolor;
  Pixmap bm;
  Status ok;
  extern char x11_xbgi_scolor_names[MAXCOLORS][64];
  int argc = XBGI->argc;
  char **argv = XBGI->argv;

  char displayname[64] = "";
  char fnname[80]      = "lucidasanstypewriter-bold-12";
  char fgname[20]      = "white";
  char bgname[20]      = "gray50";
  char defgeomstr[20]  = "640x400";
  char usrgeomstr[20]  = "640x400";
  char titlename[80]   = "";
  char iconname[80]    = "";
  char iconfile[80]    = "";
  
  XBGI->ncolor  = 16;
  XBGI->color   = XBGI->ncolor-1;
  XBGI->bkcolor = 0;
  XBGI->npage   = 2;
  XBGI->apage   = 0;
  XBGI->vpage   = 0;
  strcpy( titlename, argv[0] );
  strcpy( iconname,  argv[0] );
  XBGI->l10n = !0;
  XBGI->colornames = 0;

  /* inherit x11 of xbgi */
  *XBGI->inherit_procedures = X11_XBGI_PROCEDURES;
  XBGI->inherit_members = malloc( sizeof(x11_xbgi) );
  if ( XBGI->inherit_members == NULL )
    FatalError( "XBGI cannot inherit X of XBGI" );

  /* memory initialization of x11 */
  XBGIDisplay = NULL;
  XBGIFontStruct = NULL;
  XBGIWins = NULL;
  for ( i=DEFAULT_FONT; i<=MAX_CHAR_FONT; i++ )
    for ( j=USER_CHAR_SIZE; j<=MAX_CHAR_SIZE; j++ )
      XBGIStrokeFonts[i][j] = NULL;
  for ( j=USER_CHAR_SIZE; j<=MAX_CHAR_SIZE; j++ )
    XBGIBiFonts[j] = NULL;
  for ( j=USER_CHAR_SIZE; j<=MAX_CHAR_SIZE; j++ )
    XBGIMultiFonts[j] = NULL;

  /* get display name and initialization */
  scan_option( argc, argv, "-display %s", displayname );
  XBGIDisplay = XOpenDisplay( displayname );
  if ( XBGIDisplay == NULL ) FatalError( "X cannot connect to display" );
  XBGIColormap = DefaultColormap( XBGIDisplay, XBGIScreen );

  /* position, size, gc, font and color settings for work */
  scan_option( argc, argv, "-geometry %s", usrgeomstr );
  scan_option( argc, argv, "-fn %s", fnname );
  scan_option( argc, argv, "-fg %s", fgname );
  scan_option( argc, argv, "-bg %s", bgname );
  XBGIDefaultGC = XCreateGC( XBGIDisplay, RootWindow(XBGIDisplay,XBGIScreen),
                            0, NULL );
  XBGIFontStruct = XLoadQueryFont( XBGIDisplay, fnname );
  if ( XBGIFontStruct == NULL ) Warning( "X cannot find font structure" );
  else
    XSetFont( XBGIDisplay, XBGIDefaultGC, XBGIFontStruct->fid );
  ok = XAllocNamedColor( XBGIDisplay, XBGIColormap, fgname,
                        &fgexact, &fgcolor );
  XBGIForeground = (ok==True)?fgexact.pixel:WhitePixel(XBGIDisplay,XBGIScreen);
  ok = XAllocNamedColor( XBGIDisplay, XBGIColormap, bgname,
                        &bgexact, &bgcolor );
  XBGIBackground = (ok==True)?bgexact.pixel:BlackPixel(XBGIDisplay,XBGIScreen);
  lcolor.red   = min( bgexact.red  *5/4, 65535 );
  lcolor.green = min( bgexact.green*5/4, 65535 );
  lcolor.blue  = min( bgexact.blue *5/4, 65535 );
  dcolor.red   = max( 0, bgexact.red  *3/4 );
  dcolor.green = max( 0, bgexact.green*3/4 );
  dcolor.blue  = max( 0, bgexact.blue *3/4 );
  ok = XAllocColor( XBGIDisplay, XBGIColormap, &lcolor );
  XBGITopShadow = (ok==True)?lcolor.pixel:BlackPixel(XBGIDisplay,XBGIScreen);
  ok = XAllocColor( XBGIDisplay, XBGIColormap, &dcolor );
  XBGIBottomShadow
    = (ok==True)?dcolor.pixel:BlackPixel(XBGIDisplay,XBGIScreen);
  XSetForeground( XBGIDisplay, XBGIDefaultGC, XBGIForeground );

  /* titlename and iconname settings */
  if ( scan_option( argc, argv, "-title %s", titlename ) )
    strcpy( iconname, titlename );
  scan_option( argc, argv, "-name %s", iconname );
  scan_option( argc, argv, "-icon %s", iconfile );

  /* get user bgi settings */
  if ( (i=scan_option(argc,argv,"-gd")) )
    XBGI->gdriver = getdrivernumber( argv[i+1] );
  if ( (i=scan_option(argc,argv,"-gm")) )
    XBGI->gmode = getmodenumber( argv[i+1] );
  scan_option( argc, argv, "-ncolor %d",  &(XBGI->ncolor)  );
  scan_option( argc, argv, "-color %d",   &(XBGI->color)   );
  scan_option( argc, argv, "-bkcolor %d", &(XBGI->bkcolor) );
  scan_option( argc, argv, "-npage %d",   &(XBGI->npage)   );
  scan_option( argc, argv, "-apage %d",   &(XBGI->apage)   );
  scan_option( argc, argv, "-vpage %d",   &(XBGI->vpage)   );

  /* scanning named color list */
  if ( (j=scan_option(argc,argv,"-colornames")) ){
    c = 0;
    for ( i=j+1; i<argc; i++ ){
      if ( *argv[i] == '-' )
        break;
      strcpy( x11_xbgi_scolor_names[c++], argv[i] );
    }
    if ( 2 <= c && c<256 ){
      XBGI->colornames = !0;
      XBGI->ncolor = c;
    }
  }
  XBGI->palette.size = (unsigned char)XBGI->ncolor-1;

  /* set font valuable */
  if ( scan_option( argc, argv, "-l10n"  ) ) XBGI->l10n = !0;
  if ( scan_option( argc, argv, "-l10n-" ) ) XBGI->l10n =  0;
#ifdef ENABLE_MULTILINGUAL
  if ( setlocale( LC_ALL, "" ) == NULL )
    XBGI->l10n = 0;
  else if ( XSupportsLocale() == False )
    XBGI->l10n = 0;
  if ( !XBGI->l10n )
    XSetLocaleModifiers( "" );
#endif

  /* create windows */
  flag_mask = XGeometry( XBGIDisplay, XBGIScreen, usrgeomstr, defgeomstr,
                        0, 1, 1, 0, 0,
                        &def.x, &def.y, &def.width, &def.height );
  XBGIRoot = XCreateSimpleWindow( XBGIDisplay,
                                 RootWindow(XBGIDisplay,XBGIScreen),
                                 def.x, def.y, def.width, def.height, 0,
                                 WhitePixel(XBGIDisplay,XBGIScreen),
                                 BlackPixel(XBGIDisplay,XBGIScreen) );
  def.min_width  = 1;
  def.min_height = 1;
  def.max_width  = 2000;
  def.max_height = 2000;
  def.flags = PMinSize | PMaxSize;
  if ( flag_mask&XValue )      def.flags = def.flags | PPosition;
  if ( flag_mask&YValue )      def.flags = def.flags | USPosition;
  if ( flag_mask&WidthValue )  def.flags = def.flags | PSize;
  if ( flag_mask&HeightValue ) def.flags = def.flags | USSize;
  XSelectInput( XBGIDisplay, XBGIRoot, ExposureMask|StructureNotifyMask );
  XSetStandardProperties( XBGIDisplay, XBGIRoot,
                         titlename, iconname, None, argv, argc, &def );
  XSetWMNormalHints( XBGIDisplay, XBGIRoot, &def );

  /* input event selection, GC creation and so on */
  XBGIDepth   = DefaultDepth( XBGIDisplay, XBGIScreen );
  XBGIDrawGC  = XCreateGC( XBGIDisplay, XBGIRoot, 0, NULL );
  XBGIFillGC  = XCreateGC( XBGIDisplay, XBGIRoot, 0, NULL );
  XBGIPixelGC = XCreateGC( XBGIDisplay, XBGIRoot, 0, NULL );
  XBGIImageGC = XCreateGC( XBGIDisplay, XBGIRoot, 0, NULL );
  att.backing_store = Always;
  if ( scan_option( argc, argv, "-backing"  ) )
    att.backing_store = Always;
  if ( scan_option( argc, argv, "-backing-" ) )
    att.backing_store = NotUseful;
  XBGIWins = (Window*)malloc( XBGI->npage*sizeof(Window) );
  for ( i=0; i<XBGI->npage; i++ ){
    XBGIWins[i] = XCreateSimpleWindow( XBGIDisplay, XBGIRoot, 0, 0,
                                       def.width, def.height, 0,
                                       WhitePixel(XBGIDisplay,XBGIScreen),
                                       BlackPixel(XBGIDisplay,XBGIScreen) );
    XSelectInput( XBGIDisplay, XBGIWins[i], ExposureMask|
                 OwnerGrabButtonMask|ButtonPressMask|ButtonReleaseMask|
                 PointerMotionMask|ButtonMotionMask|KeyPressMask|
                 EnterWindowMask|LeaveWindowMask );
    XChangeWindowAttributes( XBGIDisplay, XBGIWins[i], CWBackingStore, &att );
    XSetStandardProperties( XBGIDisplay, XBGIWins[i],
                           titlename, iconname, None, argv, argc, &def );
  }
  
  /* BGI default setting */
  detectgraph( &(XBGI->gdriver), &(XBGI->gmode) );
  setgraphmode( XBGI->gmode );
  graphdefaults();

  XMapWindow( XBGIDisplay, XBGIRoot );
  XMapSubwindows( XBGIDisplay, XBGIRoot ); 
  do { /* bug fixed at version 2.3 */
    XNextEvent( XBGIDisplay, &ev );
  } while ( Expose != ev.type );
  XFlush( XBGIDisplay );

  /* set icon from data or file */
  if ( strcmp(iconfile,"")==0 ){
#if 0 /* it's bug */
    for ( i=0; xbms[i].bits != NULL; i++ ){
      bm = XCreateBitmapFromData( XBGIDisplay,
                                 RootWindow(XBGIDisplay,XBGIScreen),
                                 xbms[i].bits, xbms[i].width, xbms[i].height );
      status = XSetIconFromPixmap( XBGIDisplay, XBGIRoot, bm,
                                  (unsigned int)xbms[i].width,
                                  (unsigned int)xbms[i].height );
      XFreePixmap( XBGIDisplay, bm );
      if ( status == True )
        break;
    }
#endif
  }else{
    seticon( iconfile );
  }

  XBGI->gerror = grOk;
}

xbgi_procedures X11_XBGI_PROCEDURES = {
/* event.c ------------------------ event looper for simple event programing */
  /* common procedures
  void set_button_press_handler( int (*handler)(int button,int x,int y) );
  void set_button_release_handler( int (*handler)(int button,int x,int y) );
  void set_mouse_motion_handler( int (*handler)(int button,int x,int y) );
  void set_page_enter_handler( int (*handler)(int page) );
  void set_page_leave_handler( int (*handler)(int page) );
  void set_key_press_handler( int (*handler)(int key) );
  void set_configure_handler( int (*handler)(void) );
  void set_expose_handler( int (*handler)(void) );
  void set_noevent_handler( int (*handler)(void) );*/
  /* inheritable procedures */
  xbgi_button_press,
  xbgi_button_release,
  xbgi_mouse_motion,
  xbgi_page_enter,
  xbgi_page_leave,
  xbgi_key_press,
  xbgi_configure,
  NULL,
  xbgi_noevent,
  xbgiloop,
  isevent,
/* request.c ------------------------------------------------- request flush */
  /* inheritable procedures */
  xflush,
  xsync,
/* config.c -------------------------- configuration of graphics environment */
  /* inheritable procedures */
  getmaxx,
  getmaxy,
  setmaxx,
  setmaxy,
  getsizexy,
  setsizexy,
  getgeometry,
  setgeometry,
  settitle,
  seticonname,
  seticon,
  setbackingstore,
/* driver.c --------------------------------------- graphics driver and mode */
  /* common procedures
  char *getdriversname( int driver_number );
  int getdrivernumber( char *driver_name );
  int getmodenumber( char *mode_name );
  char *getdrivername( void );
  int getgraphmode( void );
  void setgraphmode( int gmode );
  char *getmodename( int mode_number );
  int getmaxmode( void );
  void getmoderange( int graphdriver, int *lomode, int *himode );
  unsigned setgraphbufsize( unsigned bufsize );
  void _graphfreemem( void *ptr, unsigned size );
  void *_graphgetmem( unsigned size );
  int installuserdriver( char *name, int (*detect)( void ) );
  int registerbgidriver( void (*driver)( void ) );
  int registerfarbgidriver( void *driver );*/
  /* inheritable procedures */
  detectgraph,
  _driver,
  driver,
/* init.c ------------------------------ initialize and close graphic system */
  /* common procedures
  void _initgraph( int argc, char *argv[], char *pd );
  void initgraph( int *graphdriver, int *graphmode, char *pathtodriver );
  void graphdefaults( void );
  void closegraph( void );*/
/* id.c -------------------------------- multi-graphic system using graph id */
  /* common procedures
  int graphnumber( void );
  void selectgraph( int id );
  void termgraph( int id );*/
/* util.c ------------------------------------ utilities for X-Window system */
  /* inheritable procedures */
  savegraphtoxbm,
  savegraphtoxpm,
/* error.c ------------------------------------------------- error messenger */
  /* inheritable procedures */
  graphresult,
  *grapherrormsg,
/* page.c ------------------------------------------- page as drawing device */
  /* inheritable procedures */
  cleardevice,
  setactivepage,
  setvisualpage,
  restorecrtmode,
/* viewport.c --------------------------- viewport as virtual drawing device */
  /* inheritable procedures */
  clearviewport,
  setviewport,
  getviewsettings,
/* lineinfo.c ------------------------------------------------ line settings */
  /* inheritable procedures */
  getlinesettings,
  setlinestyle,
/* writeops.c ------------------------------------------ set write operation */
  /* inheritable procedures */
  setwritemode,
/* move.c ---------------------------------------------------- current point */
  /* inheritable procedures */
  getx,
  gety,
  moverel,
  moveto,
/* line.c ----------------------------------------------------- drawing line */
  /* inheritable procedures */
  lines,
  line,
  linerel,
  lineto,
/* rect.c ------------------------------------------------ drawing rectangle */
  /* inheritable procedures */
  rectangle,
/* poly.c -------------------------------------------------- drawing polygon */
  /* inheritable procedures */
  drawpoly,
/* arcinfo.c -------------------------------------------------- arc settings */
  /* inheritable procedures */
  getarccoords,
  getaspectratio,
  setaspectratio,
/* arc.c ------------------------------------------------------- drawing arc */
  /* inheritable procedures */
  arc,
  circle,
  ellipse,
/* fillinfo.c ------------------------------------------------ fill settings */
  /* inheritable procedures */
  getfillsettings,
  getfillpattern,
  setfillstyle,
  setfillpattern,
/* fillrect.c -------------------------------------------- filling rectangle */
  /* inheritable procedures */
  bar,
  bar3d,
/* fillpoly.c ---------------------------------------------- filling polygon */
  /* inheritable procedures */
  fillpoly,
/* fillarc.c --------------------------------------------------- filling arc */
  /* inheritable procedures */
  fillellipse,
  pieslice,
  sector,
/* flood.c --------------------------------------------------- filling flood */
  /* inheritable procedures */
  floodfill,
/* pixel.c ----------------------------------------------------------- pixel */
  /* inheritable procedures */
  putpixels,
  getpixel,
  putpixel,
/* image.c ----------------------------------------------------------- image */
  /* inheritable procedures */
  imagesize,
  getimage,
  putimage,
/* textinfo.c ------------------------------------------------ text settings */
  /* inheritable procedures */
  textheight,
  textwidth,
  gettextsettings,
  settextstyle,
  settextjustify,
  setusercharsize,
  installuserfont,
  registerbgifont,
  registerfarbgifont,
/* text.c --------------------------------------------- out text on graphics */
  /* inheritable procedures */
  outtext,
  outtextxy,
/* color.c ----------------------------------------------- get and set color */
  /* inheritable procedures */
  getcolor,
  getbkcolor,
  getmaxcolor,
  setcolor,
  setbkcolor,
  setmaxcolor,
/* palette.c -------------------------------- get and set palette and colors */
  /* inheritable procedures */
  *getdefaultpalette,
  getpalette,
  getpalettesize,
  setallpalette,
  setpalette,
  setrgbpalette,
  _setrgbpalette,
/* bgiutl.c ----------------------------------- meta-level procedures of bgi */
  /* inheritable procedures */
  bgivprintf,
  bgiprintf,
  bgigets,
  bgimenu,
/* kbhit.c ------------------------------------------ get keyboard hit state */
  /* inheritable procedures */
  kbhit,
/* getch.c ---------------------------------------------- key pressed return */
  /* inheritable procedures */
  getch,
/* conio.c ---------------------------------------------- console procedures */
  /* all common procedures, reserved future */
/* ------------------------------------------------------------------------- */
};

#include <X11/Intrinsic.h>
int x11_widget_getmaxx( void )
{
  Window root;
  int x, y, width, height, border, depth;
  
  XGetGeometry( XBGIDisplay, XBGIWins[XBGI->apage], &root,
	       &x, &y, &width, &height, &border, &depth );
  return width-1;
}
int x11_widget_getmaxy( void )
{
  Window root;
  int x, y, width, height, border, depth;
  
  XGetGeometry( XBGIDisplay, XBGIWins[XBGI->apage], &root,
	       &x, &y, &width, &height, &border, &depth );
  return height-1;
}
void x11_widget_setmaxx( int maxx )
{
#if 0
  Window root;
  int i, x, y, width, height, border, depth;
  
  if ( maxx <= 0 ) return;
  XGetGeometry( XBGIDisplay, XBGIWins[XBGI->apage], &root,
	       &x, &y, &width, &height, &border, &depth );
  XResizeWindow( XBGIDisplay, XBGIWins[XBGI->apage], maxx+1, height );
#endif
}
void x11_widget_setmaxy( int maxy )
{
#if 0
  Window root;
  int i, x, y, width, height, border, depth;

  if ( maxy <= 0 ) return;
  XGetGeometry( XBGIDisplay, XBGIWins[XBGI->apage], &root,
	       &x, &y, &width, &height, &border, &depth );
  XResizeWindow( XBGIDisplay, XBGIWins[XBGI->apage], width, maxy+1 );
#endif
}
void x11_widget_getsizexy( int *xsize, int *ysize )
{
  Window root;
  int i, x, y, width, height, border, depth;
  
  XGetGeometry( XBGIDisplay, XBGIWins[XBGI->apage], &root,
	       &x, &y, &width, &height, &border, &depth );
  *xsize = width;
  *ysize = height;
}
void x11_widget_setsizexy( int xsize, int ysize )
{
#if 0
  int i;
  
  if ( xsize <= 0 || ysize <= 0 ) return;
  XResizeWindow( XBGIDisplay, XBGIWins[XBGI->apage], xsize, ysize );
#endif
}
void x11_widget_getgeometry( int *x, int *y, int *width, int *height )
{
  Window root;
  int border, depth;
  
  XGetGeometry( XBGIDisplay, XBGIWins[XBGI->apage], &root,
               x, y, width, height, &border, &depth );
}
void x11_widget_setgeometry( int x, int y, int width, int height )
{
#if 0
  XMoveWindow( XBGIDisplay, XBGIWins[XBGI->apage], x, y );
  XResizeWindow( XBGIDisplay, XBGIWins[XBGI->apage], width, height );
#endif
}
void x11_widget__driver( void )
{
  XFreeGC( XBGIDisplay, XBGIDrawGC  );
  XFreeGC( XBGIDisplay, XBGIFillGC  );
  XFreeGC( XBGIDisplay, XBGIPixelGC );
  XFreeGC( XBGIDisplay, XBGIImageGC );

  XFreeColormap( XBGIDisplay, XBGIColormap );
  if ( XBGIFontStruct ) XFreeFont( XBGIDisplay, XBGIFontStruct );

#if 0 /* for segmentation fault bug? */
  free( XBGI->inherit_members );
  XBGI->inherit_members = NULL;
#endif
}
xbgi *XbgiSimpleCreateFromWidget( Widget w )
{
  int i, j;
  extern xbgi *XBGI;
  XSetWindowAttributes att;
  
  att.backing_store = Always;
  XBGI = (xbgi*)malloc(sizeof(xbgi));
  XBGI->gdriver = X11;
  XBGI->gmode = X11COLOR;
  XBGI->gerror = grNoInitGraph;
  XBGI->ncolor = 16;
  XBGI->color = XBGI->ncolor-1;
  XBGI->bkcolor = 0;
  XBGI->npage = 1;
  XBGI->apage = 0;
  XBGI->vpage = 0;
  XBGI->l10n = !0;
  XBGI->colornames = 0;
  XBGI->palette.size = (unsigned char)XBGI->ncolor-1;

  XBGI->inherit_procedures
    = (xbgi_procedures*)malloc(sizeof(xbgi_procedures));
  *XBGI->inherit_procedures = X11_XBGI_PROCEDURES;
#undef getmaxx
#undef getmaxy
#undef setmaxx
#undef setmaxy
#undef setsizexy
#undef getsizexy
#undef setgeometry
#undef getgeometry
#undef _driver
  XBGI->inherit_procedures->getmaxx = x11_widget_getmaxx;
  XBGI->inherit_procedures->getmaxy = x11_widget_getmaxy;
  XBGI->inherit_procedures->setmaxx = x11_widget_setmaxx;
  XBGI->inherit_procedures->setmaxy = x11_widget_setmaxy;
  XBGI->inherit_procedures->setsizexy = x11_widget_setsizexy;
  XBGI->inherit_procedures->getsizexy = x11_widget_getsizexy;
  XBGI->inherit_procedures->setgeometry = x11_widget_setgeometry;
  XBGI->inherit_procedures->getgeometry = x11_widget_getgeometry;
  XBGI->inherit_procedures->_driver = x11_widget__driver;
  XBGI->inherit_members = malloc( sizeof(x11_xbgi) );

  XBGIDisplay = XtDisplay( w );
  XBGIColormap = DefaultColormap( XBGIDisplay, XBGIScreen );
  XBGIDefaultGC =
    XCreateGC( XBGIDisplay, RootWindow(XBGIDisplay,XBGIScreen), 0, NULL );
  XBGIDepth = DefaultDepth( XBGIDisplay, XBGIScreen );
  XBGIRoot = RootWindow(XBGIDisplay,XBGIScreen);
  XBGIDrawGC = XCreateGC( XBGIDisplay, XBGIRoot, 0, NULL );
  XBGIFillGC = XCreateGC( XBGIDisplay, XBGIRoot, 0, NULL );
  XBGIPixelGC = XCreateGC( XBGIDisplay, XBGIRoot, 0, NULL );
  XBGIImageGC = XCreateGC( XBGIDisplay, XBGIRoot, 0, NULL );
  XBGIWins = (Window*)malloc( XBGI->npage*sizeof(Window) );
  XBGIWins[0] = XtWindow( w );
  XSelectInput( XBGIDisplay, XBGIWins[0], ExposureMask|
	       OwnerGrabButtonMask|ButtonPressMask|ButtonReleaseMask|
	       PointerMotionMask|ButtonMotionMask|KeyPressMask|
	       EnterWindowMask|LeaveWindowMask );
  XChangeWindowAttributes( XBGIDisplay, XBGIWins[0], CWBackingStore, &att );
  for ( i=DEFAULT_FONT; i<=MAX_CHAR_FONT; i++ )
    for ( j=USER_CHAR_SIZE; j<=MAX_CHAR_SIZE; j++ )
      XBGIStrokeFonts[i][j] = NULL;
  for ( j=USER_CHAR_SIZE; j<=MAX_CHAR_SIZE; j++ )
    XBGIBiFonts[j] = NULL;
  for ( j=USER_CHAR_SIZE; j<=MAX_CHAR_SIZE; j++ )
    XBGIMultiFonts[j] = NULL;
  setmaxcolor( XBGI->ncolor-1 );
  graphdefaults();
  XBGI->gerror = grOk;
  return XBGI;
}
xbgi *XbgiMonoCreateFromWidget( Widget w )
{
  int i, j;
  extern xbgi *XBGI;
  XSetWindowAttributes att;
  
  att.backing_store = Always;
  XBGI = (xbgi*)malloc(sizeof(xbgi));
  XBGI->gdriver = X11;
  XBGI->gmode = X11COLOR;
  XBGI->gerror = grNoInitGraph;
  XBGI->ncolor = 2;
  XBGI->color = XBGI->ncolor-1;
  XBGI->bkcolor = 0;
  XBGI->npage = 1;
  XBGI->apage = 0;
  XBGI->vpage = 0;
  XBGI->l10n = !0;
  XBGI->colornames = 0;
  XBGI->palette.size = (unsigned char)XBGI->ncolor-1;

  XBGI->inherit_procedures
    = (xbgi_procedures*)malloc(sizeof(xbgi_procedures));
  *XBGI->inherit_procedures = X11_XBGI_PROCEDURES;
#undef getmaxx
#undef getmaxy
#undef setmaxx
#undef setmaxy
#undef setsizexy
#undef getsizexy
#undef setgeometry
#undef getgeometry
#undef _driver
  XBGI->inherit_procedures->getmaxx = x11_widget_getmaxx;
  XBGI->inherit_procedures->getmaxy = x11_widget_getmaxy;
  XBGI->inherit_procedures->setmaxx = x11_widget_setmaxx;
  XBGI->inherit_procedures->setmaxy = x11_widget_setmaxy;
  XBGI->inherit_procedures->setsizexy = x11_widget_setsizexy;
  XBGI->inherit_procedures->getsizexy = x11_widget_getsizexy;
  XBGI->inherit_procedures->setgeometry = x11_widget_setgeometry;
  XBGI->inherit_procedures->getgeometry = x11_widget_getgeometry;
  XBGI->inherit_procedures->_driver = x11_widget__driver;
  XBGI->inherit_members = malloc( sizeof(x11_xbgi) );

  XBGIDisplay = XtDisplay( w );
  XBGIColormap = DefaultColormap( XBGIDisplay, XBGIScreen );
  XBGIDefaultGC =
    XCreateGC( XBGIDisplay, RootWindow(XBGIDisplay,XBGIScreen), 0, NULL );
  XBGIDepth = DefaultDepth( XBGIDisplay, XBGIScreen );
  XBGIRoot = RootWindow(XBGIDisplay,XBGIScreen);
  XBGIDrawGC = XCreateGC( XBGIDisplay, XBGIRoot, 0, NULL );
  XBGIFillGC = XCreateGC( XBGIDisplay, XBGIRoot, 0, NULL );
  XBGIPixelGC = XCreateGC( XBGIDisplay, XBGIRoot, 0, NULL );
  XBGIImageGC = XCreateGC( XBGIDisplay, XBGIRoot, 0, NULL );
  XBGIWins = (Window*)malloc( XBGI->npage*sizeof(Window) );
  XBGIWins[0] = XtWindow( w );
  XSelectInput( XBGIDisplay, XBGIWins[0], ExposureMask|
	       OwnerGrabButtonMask|ButtonPressMask|ButtonReleaseMask|
	       PointerMotionMask|ButtonMotionMask|KeyPressMask|
	       EnterWindowMask|LeaveWindowMask );
  XChangeWindowAttributes( XBGIDisplay, XBGIWins[0], CWBackingStore, &att );
  for ( i=DEFAULT_FONT; i<=MAX_CHAR_FONT; i++ )
    for ( j=USER_CHAR_SIZE; j<=MAX_CHAR_SIZE; j++ )
      XBGIStrokeFonts[i][j] = NULL;
  for ( j=USER_CHAR_SIZE; j<=MAX_CHAR_SIZE; j++ )
    XBGIBiFonts[j] = NULL;
  for ( j=USER_CHAR_SIZE; j<=MAX_CHAR_SIZE; j++ )
    XBGIMultiFonts[j] = NULL;
  setmaxcolor( XBGI->ncolor-1 );
  graphdefaults();
  XBGI->gerror = grOk;
  return XBGI;
}



